var map;
var layer_temp;
var layer_precipitation;

const form = document.querySelector("form");
form.addEventListener("submit", onFormSubmit);

const clearButton = document.querySelector('#clearbutton');
clearButton.addEventListener('click', onClearButtonClick);

const logButton = document.querySelector('#logbutton');
logButton.addEventListener('click', onLogButtonClick);

// data validation
function onFormSubmit(event) {
  event.preventDefault();
  var isValid = true;
  const address = document.querySelector("#address");
  const region = document.querySelector("#region");
  const city = document.querySelector("#city");

  if (address.value === "") {
    const addressAlert = document.querySelector("#addressAlert");
    addressAlert.classList.remove("d-none");
	isValid = false;
  } else {
    const addressAlert = document.querySelector("#addressAlert");
    addressAlert.classList.add("d-none");
  }
  if (region.value === "") {
    const regionAlert = document.querySelector("#regionAlert");
    regionAlert.classList.remove("d-none");
	isValid = false;
  } else {
    const regionAlert = document.querySelector("#regionAlert");
    regionAlert.classList.add("d-none");
  }
  if (city.options[city.selectedIndex].text === "Select city") { // (city.options[city.selectedIndex].value === 0)
    const cityAlert = document.querySelector("#cityAlert");
    cityAlert.classList.remove("d-none");
	isValid = false;
  } else {
    const cityAlert = document.querySelector("#cityAlert");
    cityAlert.classList.add("d-none");
  }
  
  if(isValid) {
	  const celsius = document.querySelector("#celsius");
	  var units;
	  if(celsius.checked === true)
		  units = 'metric';
	  else
		  units = 'imperial';
	  // save request data into database 
	  setLog(address.value, region.value, city.options[city.selectedIndex].text);
	  // translate address to gps coordinates
	  getGeocoding(address.value, region.value, city.options[city.selectedIndex].text, units);
  }
}

// clear user input and reset select and radio button
function onClearButtonClick(event) {
	// clear inputs
	const address = document.querySelector("#address");
	address.value = "";
	const region = document.querySelector("#region");
	region.value = "";
	// reset selected value
	const city = document.querySelector("#city");
	city.selectedIndex = 0;
	// remove error messages
	const addressAlert = document.querySelector("#addressAlert");
    addressAlert.classList.add("d-none");
	const regionAlert = document.querySelector("#regionAlert");
    regionAlert.classList.add("d-none");
	const cityAlert = document.querySelector("#cityAlert");
    cityAlert.classList.add("d-none");
	// reset to celsius
	const celsius = document.querySelector("#celsius");
	celsius.checked = true; 
    const fahrenheit = document.querySelector("#fahrenheit");
	fahrenheit.checked = false; 
	
	// disable results section
	const results = document.querySelector("#results");
	results.classList.add("d-none");
	// clear table next 24h
	const tbody = document.querySelector('#tbody24h');
	tbody.innerHTML = '';
	// remove map (if it exists)
	if (map){
		map.removeLayer(layer_temp);
		map.removeLayer(layer_precipitation);
		map.setTarget(null);
		map = null;
	}
}

function onLogButtonClick(event) {
	// Set up our HTTP request
	var xhr = new XMLHttpRequest();

	// Setup our listener to process compeleted requests
	xhr.onreadystatechange = function () {

		// Only run if the request is complete
		if (xhr.readyState !== 4) return;

		// Process our return data
		if (xhr.status >= 200 && xhr.status < 300) {
			// What do when the request is successful
			jsonObj = xhr.response;
			const tbody = document.querySelector('#tbodylog');
			tbody.innerHTML = '';
			for(let i=0; i<jsonObj.length; i++) {
				const tr = document.createElement('tr');
				const td1 = document.createElement('td');
				td1.textContent = dateConverter(jsonObj[i].timestamp);
				tr.appendChild(td1);
				const td2 = document.createElement('td');
				td2.textContent = jsonObj[i].address;
				tr.appendChild(td2);
				const td3 = document.createElement('td');
				td3.textContent = jsonObj[i].region;
				tr.appendChild(td3);
				const td4 = document.createElement('td');
				td4.textContent = jsonObj[i].city;
				tr.appendChild(td4);
				tbody.appendChild(tr);
			}
		} else {
			// What to do when the request has failed
			console.log('error', xhr);
		}

	};

	// Create and send a GET request
	// The first argument is the post type (GET, POST, PUT, DELETE, etc.)
	// The second argument is the endpoint URL
	xhr.open('GET', 'logdata.php?username=csp5pa1');
	// setting the responseType to JSON, so that XHR knows that the server will be returning JSON
	xhr.responseType = 'json';
	xhr.send();
}

function setLog(address, region, city) {
	// Set up our HTTP request
	var xhr = new XMLHttpRequest();

	// Setup our listener to process compeleted requests
	xhr.onreadystatechange = function () {

		// Only run if the request is complete
		if (xhr.readyState !== 4) return;

		// Process our return data
		if (xhr.status >= 200 && xhr.status < 300) {
			//console.log(xhr.status);
		} else {
			// What to do when the request has failed
			console.log('error', xhr);
		}
	};

	// Create and send a POST request
	// The first argument is the post type (GET, POST, PUT, DELETE, etc.)
	// The second argument is the endpoint URL
	xhr.open('POST', "logdata.php");
	// setting the content-type header to application/json, so that the server will know the data type
	xhr.setRequestHeader("Content-Type", "application/json");
	
	var data = {};
	data.address = address;
	data.region = region;
	data.city = city;
	data.username = "csp5pa1";
	
	xhr.send(JSON.stringify(data));
}

function getGeocoding(address, region, city, units) {
	const url = 'https://nominatim.openstreetmap.org/search?q=' + address + ',' + region + ',' + city + '&format=json';
	
	// Set up our HTTP request
	var xhr = new XMLHttpRequest();

	// Setup our listener to process compeleted requests
	xhr.onreadystatechange = function () {

		// Only run if the request is complete
		if (xhr.readyState !== 4) return;

		// Process our return data
		if (xhr.status >= 200 && xhr.status < 300) {
			// What do when the request is successful
			jsonObj = xhr.response;
			//console.log(jsonObj);
			
			if(jsonObj === undefined || jsonObj.length == 0) {
				alert("No result for that location.");
			}
			else {
				// get attraction info from OpenAI
	  			getAttractionInfo(city);

				const latitude = parseFloat(jsonObj[0].lat);
				const longitude = parseFloat(jsonObj[0].lon);
				//console.log(latitude);
				//console.log(longitude);

				// enable results section 
				// (for Open layers map rendering -- problem if section is enabled after)
				const results = document.querySelector("#results");
				results.classList.remove("d-none");
				
				getCurrentWeather(latitude, longitude, units);
				// clear table next 24h (if there are data from previous call)
				const tbody = document.querySelector('#tbody24h');
				tbody.innerHTML = '';
				// get weather forecast
				getWeatherForecast(latitude, longitude, units);
				
				// remove map (if it exists)
				if (map){
					map.removeLayer(layer_temp);
					map.removeLayer(layer_precipitation);
					map.setTarget(null);
					map = null;
				}
				showMap(latitude, longitude);
			}
			
		} else {
			// What to do when the request has failed
			console.log('error', xhr);
		}

	};

	// Create and send a GET request
	// The first argument is the post type (GET, POST, PUT, DELETE, etc.)
	// The second argument is the endpoint URL
	xhr.open('GET', url);
	// setting the responseType to JSON, so that XHR knows that the server will be returning JSON
	xhr.responseType = 'json';
	xhr.send();
}


function getAttractionInfo(city) {
	OPENAI_ID = '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';
	var data = {};
	data.model = "text-davinci-003";
	data.prompt = "Give information about three important attractions a tourist can see in " + city + ". Format it as html list.";
	data.temperature = 0.7;
	data.max_tokens = 256;
	data.top_p = 1;
	data.frequency_penalty = 0;
	data.presence_penalty = 0;

	// disable spinner section
	const spinner = document.querySelector("#spinner");
	spinner.classList.remove("d-none");
	const attractions = document.querySelector('#attractions');
	attractions.textContent = "";

	fetch('https://api.openai.com/v1/completions', {
		method: 'POST',
		headers: {
			'Content-Type': 'application/json',
			'Authorization': 'Bearer ' + OPENAI_ID 
		},
		body: JSON.stringify(data),
	})
	.then((response) => response.json())
	.then((data) => {
		console.log('Success:', data);
		const cityname = document.querySelector('#cityname');
		cityname.textContent = city;
		
		attractions.innerHTML = data.choices[0].text;
		spinner.classList.add("d-none");
	})
	.catch((error) => {
		console.error('Error:', error);
	});
}

/* GOOGLE GEOCODING IS NOT FREE
function getGeocoding_Google(address, region, city, units) {
	API_KEY = 'AIzaSyBolqMraIH7sVr4HWzhEmx0rKgamh-MZz0';
	const url = 'https://maps.googleapis.com/maps/api/geocode/json?address=' + address + '+' + region + '+' + city + '&key=' + API_KEY;
	
	// Set up our HTTP request
	var xhr = new XMLHttpRequest();

	// Setup our listener to process compeleted requests
	xhr.onreadystatechange = function () {

		// Only run if the request is complete
		if (xhr.readyState !== 4) return;

		// Process our return data
		if (xhr.status >= 200 && xhr.status < 300) {
			// What do when the request is successful
			jsonObj = xhr.response;
			const latitude = jsonObj['results'][0].geometry.location.lat;
			const longitude = jsonObj['results'][0].geometry.location.lng;
			//console.log(latitude);
			//console.log(longitude);
			
			// enable results section 
			// (for Open layers map rendering -- problem if section is enabled after)
			const results = document.querySelector("#results");
			results.classList.remove("d-none");
			
			getCurrentWeather(latitude, longitude, units);
			// clear table next 24h (if there are data from previous call)
			const tbody = document.querySelector('#tbody24h');
			tbody.innerHTML = '';
			// get weather forecast
			getWeatherForecast(latitude, longitude, units);
			
			// remove map (if it exists)
			if (map){
				map.removeLayer(layer_temp);
				map.removeLayer(layer_precipitation);
				map.setTarget(null);
				map = null;
			}
			showMap(latitude, longitude);
			
		} else {
			// What to do when the request has failed
			console.log('error', xhr);
		}

	};

	// Create and send a GET request
	// The first argument is the post type (GET, POST, PUT, DELETE, etc.)
	// The second argument is the endpoint URL
	xhr.open('GET', url);
	// setting the responseType to JSON, so that XHR knows that the server will be returning JSON
	xhr.responseType = 'json';
	xhr.send();
}*/

function getCurrentWeather(lat, lon, units) {
	APP_ID = '1b2b3a82a8e633366caef05c1f70bcb2';
	const url = 'https://api.openweathermap.org/data/2.5/weather?lat=' + lat + '&lon=' + lon + '&units=' + units + '&APPID=' + APP_ID;
	
	// Set up our HTTP request
	var xhr = new XMLHttpRequest();

	// Setup our listener to process compeleted requests
	xhr.onreadystatechange = function () {

		// Only run if the request is complete
		if (xhr.readyState !== 4) return;

		// Process our return data
		if (xhr.status >= 200 && xhr.status < 300) {
			// What do when the request is successful
			jsonObj = xhr.response;
			
			icon = 'https://openweathermap.org/img/w/' + jsonObj.weather[0].icon + '.png';
			desc = jsonObj.weather[0].description;
			loc = jsonObj.name;
			
			const img = document.querySelector('#icon');
			img.src = icon;
			const description = document.querySelector("#description");
			description.textContent = desc;
			const place = document.querySelector("#place");
			place.textContent = loc;
			
			var temp = jsonObj.main.temp;
			var temp_min = jsonObj.main.temp_min;
			var temp_max = jsonObj.main.temp_max;
			if (units === 'metric') {
				temp += ' <sup>o</sup>C';
				temp_min += ' <sup>o</sup>C';
				temp_max += ' <sup>o</sup>C';
			}
			else {
				temp += ' <sup>o</sup>F';
				temp_min += ' <sup>o</sup>F';
				temp_max += ' <sup>o</sup>F';
			}
			var curtemp = document.querySelector("#curtemp");
			curtemp.innerHTML = temp;
			var mintemp = document.querySelector("#mintemp");
			mintemp.innerHTML = 'L:' + temp_min;
			var maxtemp = document.querySelector("#maxtemp");
			maxtemp.innerHTML = 'H:' + temp_max;
			
			var pressure = jsonObj.main.pressure;
			if (units === 'metric')
				pressure += ' hPa';
			else
				pressure += ' mb';
			var humidity = jsonObj.main.humidity + ' %';
			var windspeed = jsonObj.wind.speed;
			if (units === 'metric')
				windspeed += ' meters/sec';
			else
				windspeed += ' miles/sec';
			var cloudiness = jsonObj.clouds.all + ' %';
			var sunrise = timeConverter(jsonObj.sys.sunrise);
			var sunset = timeConverter(jsonObj.sys.sunset);			
			
			const pressureId = document.querySelector("#pressure");
			pressureId.textContent = pressure;
			const humidityId = document.querySelector("#humidity");
			humidityId.textContent = humidity;
			const windspeedId = document.querySelector("#windspeed");
			windspeedId.textContent = windspeed;
			const cloudinessId = document.querySelector("#cloudiness");
			cloudinessId.textContent = cloudiness;
			const sunriseId = document.querySelector("#sunrise");
			sunriseId.textContent = sunrise;
			const sunsetId = document.querySelector("#sunset");
			sunsetId.textContent = sunset;
		} else {
			// What to do when the request has failed	
			console.log('error', xhr);
		}

	};

	// Create and send a GET request
	// The first argument is the post type (GET, POST, PUT, DELETE, etc.)
	// The second argument is the endpoint URL
	xhr.open('GET', url);
	// setting the responseType to JSON, so that XHR knows that the server will be returning JSON
	xhr.responseType = 'json';
	xhr.send();
}

function getWeatherForecast(lat, lon, units) {
	APP_ID = '1b2b3a82a8e633366caef05c1f70bcb2';
	const url = 'https://api.openweathermap.org/data/2.5/forecast?lat=' + lat + '&lon=' + lon + '&units=' + units + '&APPID=' + APP_ID;
	
	// Set up our HTTP request
	var xhr = new XMLHttpRequest();

	// Setup our listener to process compeleted requests
	xhr.onreadystatechange = function () {

		// Only run if the request is complete
		if (xhr.readyState !== 4) return;

		// Process our return data
		if (xhr.status >= 200 && xhr.status < 300) {
			// What do when the request is successful
			jsonObj = xhr.response;
			const tbody = document.querySelector('#tbody24h');
			for(let i=0; i<jsonObj.list.length/5; i++) {
				const tr = document.createElement('tr');
				const td1 = document.createElement('td');
				td1.textContent = timeConverter(jsonObj.list[i].dt);
				tr.appendChild(td1);
				const td2 = document.createElement('td');
				const img = document.createElement('img');
				img.src = 'http://openweathermap.org/img/w/' + jsonObj.list[i].weather[0].icon + '.png';
				td2.appendChild(img);
				tr.appendChild(td2);
				const td3 = document.createElement('td');
				var temp = jsonObj.list[i].main.temp;
				if (units === 'metric') {
					temp += ' <sup>o</sup>C';
				}
				else {
					temp += ' <sup>o</sup>F';
				}
				td3.innerHTML = temp;
				tr.appendChild(td3);
				const td4 = document.createElement('td');
				td4.textContent = jsonObj.list[i].clouds.all + ' %';
				tr.appendChild(td4);
				const td5 = document.createElement('td');
				const btn = document.createElement('button');
				btn.setAttribute('type', 'button');
				btn.classList.add('btn');
				btn.classList.add('btn-success');
				btn.setAttribute('data-bs-toggle', 'modal');
				btn.setAttribute('data-bs-target', '#modal' + i);
				btn.textContent = 'View';
				td5.appendChild(btn);
				const modalDiv = document.createElement('div');
				modalDiv.innerHTML = "<div class='modal fade' id='modal" + i +"' tabindex='-1' role='dialog' aria-labelledby='modal" + i + "label' aria-hidden='true'> \
				  <div class='modal-dialog modal-dialog-centered' role='document'> \
					<div class='modal-content'> \
					  <div class='modal-header'> \
						<h5 class='modal-title' id='modal" + i + "label'></h5> \
						<button type='button' class='btn-close' data-bs-dismiss='modal' aria-label='Close'> \
						</button> \
					  </div> \
					  <div class='modal-body'> \
						<div class='row'> \
							<div class='col-6 col-sm-6 text-center'><img id='modal" + i + "icon'/></div> \
							<div class='col-6 col-sm-6 text-center margin-top-15' id='modal" + i + "description'></div> \
						</div> \
						<div class='row'> \
							<div class='col-4 col-sm-4 text-center'><strong>Humidity</strong></div> \
							<div class='col-4 col-sm-4 text-center'><strong>Pressure</strong></div> \
							<div class='col-4 col-sm-4 text-center'><strong>Wind Speed</strong></div> \
						</div> \
						<div class='row'> \
							<div class='col-4 col-sm-4 text-center' id='modal" + i + "humidity'></div> \
							<div class='col-4 col-sm-4 text-center' id='modal" + i + "pressure'></div> \
							<div class='col-4 col-sm-4 text-center' id='modal" + i + "windspeed'></div> \
						</div> \
					  </div> \
					  <div class='modal-footer'> \
						<button type='button' class='btn btn-danger' data-bs-dismiss='modal'>Close</button> \
					  </div> \
					</div> \
				  </div> \
				</div>";
				td5.appendChild(modalDiv);
				tr.appendChild(td5);
				tbody.appendChild(tr);
				const modallabel = document.querySelector('#modal' + i + 'label');
				modallabel.textContent = 'Weather in ' + jsonObj.city.name + ' on ' + dateConverter(jsonObj.list[i].dt);
				const modalimage = document.querySelector('#modal' + i + 'icon');
				modalimage.src = 'http://openweathermap.org/img/w/' + jsonObj.list[i].weather[0].icon + '.png';
				const modaldescription = document.querySelector('#modal' + i + 'description');
				modaldescription.textContent = jsonObj.list[i].weather[0].main + ' (' + jsonObj.list[i].weather[0].description + ')';
				const modalhumidity = document.querySelector('#modal' + i + 'humidity');
				modalhumidity.textContent = jsonObj.list[i].main.humidity + ' %';
				const modalpressure = document.querySelector('#modal' + i + 'pressure');
				var pressure = jsonObj.list[i].main.pressure;
				if (units === 'metric')
					pressure += ' hPa';
				else
					pressure += ' mb';
				modalpressure.textContent = pressure;
				const modalwindspeed = document.querySelector('#modal' + i + 'windspeed');
				var windspeed = jsonObj.list[i].wind.speed;
				if (units === 'metric')
					windspeed += ' meters/sec';
				else
					windspeed += ' miles/sec';
				modalwindspeed.textContent = windspeed;
			}
		} else {
			// What to do when the request has failed	
			console.log('error', xhr);
		}

	};

	// Create and send a GET request
	// The first argument is the post type (GET, POST, PUT, DELETE, etc.)
	// The second argument is the endpoint URL
	xhr.open('GET', url);
	// setting the responseType to JSON, so that XHR knows that the server will be returning JSON
	xhr.responseType = 'json';
	xhr.send();
}

function showMap(lat, lon) {
	map = new ol.Map({
        target: 'mapid',
        layers: [
          new ol.layer.Tile({
            source: new ol.source.OSM()
          })
        ],
        view: new ol.View({
          center: ol.proj.fromLonLat([lon, lat]),
          zoom: 5
        })
    });
	  
    layer_temp = new ol.layer.Tile({
          source: new ol.source.XYZ({
            // Replace this URL with a URL you generate. To generate an ID go to http://home.openweathermap.org/
            // and click "map editor" in the top right corner. Make sure you're registered!
            url: 'https://tile.openweathermap.org/map/temp_new/{z}/{x}/{y}.png?appid=1b2b3a82a8e633366caef05c1f70bcb2',
          })
    });
    map.addLayer(layer_temp);
	layer_precipitation = new ol.layer.Tile({
          source: new ol.source.XYZ({
            // Replace this URL with a URL you generate. To generate an ID go to http://home.openweathermap.org/
            // and click "map editor" in the top right corner. Make sure you're registered!
            url: 'https://tile.openweathermap.org/map/precipitation_new/{z}/{x}/{y}.png?appid=1b2b3a82a8e633366caef05c1f70bcb2',
          })
    });
	map.addLayer(layer_precipitation);
}

function timeConverter(unix_timestamp) {
	// Create a new JavaScript Date object based on the timestamp
	// multiplied by 1000 so that the argument is in milliseconds, not seconds.
	var date = new Date(unix_timestamp*1000);
	// Minutes part from the timestamp
	var minutes = (date.getMinutes() < 10 ? '0' : '') + date.getMinutes();
	// Hours part from the timestamp
	var hours = (date.getHours() < 10 ? '0' : '') + date.getHours();	
	/*var hours;
	var isAM = true;
	if (date.getHours() > 12) {
		if (date.getHours() - 12 < 10)
			hours = '0' + (date.getHours() - 12);
		else
			hours = (date.getHours() - 12);
		isAM = false;
	}
	else
		if (date.getHours() < 10)
			hours = '0' + date.getHours();
		else
			hours = date.getHours();*/
	// Seconds part from the timestamp
	//var seconds = (date.getSeconds() < 10 ? '0' : '') + date.getSeconds();

	// Will display time in 10:30:23 format
	//return formattedTime = hours + ':' + minutes.substr(-2) + (isAM ? ' AM' : ' PM'); // + ':' + seconds.substr(-2);
	return formattedTime = hours + ':' + minutes.substr(-2); // + ':' + seconds.substr(-2);
}

function dateConverter(unix_timestamp){
  var a = new Date(unix_timestamp * 1000);
  var months = ['Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec'];
  var year = a.getFullYear();
  var month = months[a.getMonth()];
  var date = a.getDate();
  var hours = a.getHours() < 10 ? '0' + a.getHours() : a.getHours();
  var minutes = a.getMinutes() < 10 ? '0' + a.getMinutes() : a.getMinutes();
  //var seconds = a.getSeconds() < 10 ? '0' + a.getSeconds() : a.getSeconds();
  var time = date + ' ' + month + ' ' + year + ' ' + hours + ':' + minutes;// + ':' + sec ;
  return time;
}